package sun.java2d.d3d;

import sun.java2d.pipe.BufferedBufImgOps;
import java.awt.image.BufferedImageOp;
import java.awt.image.BufferedImage;
import sun.java2d.ScreenUpdateManager;
import sun.java2d.pipe.BufferedContext;
import java.awt.geom.AffineTransform;
import sun.java2d.pipe.Region;
import java.awt.Composite;
import sun.java2d.pipe.RenderBuffer;
import sun.java2d.SurfaceData;
import sun.java2d.pipe.RenderQueue;
import sun.java2d.loops.GraphicsPrimitiveMgr;
import sun.java2d.loops.CompositeType;
import sun.java2d.loops.GraphicsPrimitive;
import sun.java2d.loops.SurfaceType;

final class D3DBlitLoops
{
    private static final int OFFSET_SRCTYPE = 16;
    private static final int OFFSET_HINT = 8;
    private static final int OFFSET_TEXTURE = 3;
    private static final int OFFSET_RTT = 2;
    private static final int OFFSET_XFORM = 1;
    private static final int OFFSET_ISOBLIT = 0;
    
    D3DBlitLoops() {
    }
    
    static void register() {
        D3DSwToSurfaceBlit d3DSwToSurfaceBlit = new D3DSwToSurfaceBlit(SurfaceType.IntArgbPre, 1);
        D3DSwToTextureBlit d3DSwToTextureBlit = new D3DSwToTextureBlit(SurfaceType.IntArgbPre, 1);
        D3DSwToSurfaceTransform d3DSwToSurfaceTransform = new D3DSwToSurfaceTransform(SurfaceType.IntArgbPre, 1);
        GraphicsPrimitiveMgr.register(new GraphicsPrimitive[] { new D3DSurfaceToGDIWindowSurfaceBlit(), new D3DSurfaceToGDIWindowSurfaceScale(), new D3DSurfaceToGDIWindowSurfaceTransform(), new D3DSurfaceToSurfaceBlit(), new D3DSurfaceToSurfaceScale(), new D3DSurfaceToSurfaceTransform(), new D3DRTTSurfaceToSurfaceBlit(), new D3DRTTSurfaceToSurfaceScale(), new D3DRTTSurfaceToSurfaceTransform(), new D3DSurfaceToSwBlit(SurfaceType.IntArgb, 0), d3DSwToSurfaceBlit, new D3DSwToSurfaceBlit(SurfaceType.IntArgb, 0), new D3DSwToSurfaceBlit(SurfaceType.IntRgb, 3), new D3DSwToSurfaceBlit(SurfaceType.IntBgr, 4), new D3DSwToSurfaceBlit(SurfaceType.ThreeByteBgr, 9), new D3DSwToSurfaceBlit(SurfaceType.Ushort565Rgb, 5), new D3DSwToSurfaceBlit(SurfaceType.Ushort555Rgb, 6), new D3DSwToSurfaceBlit(SurfaceType.ByteIndexed, 7), new D3DGeneralBlit(D3DSurfaceData.D3DSurface, CompositeType.AnyAlpha, d3DSwToSurfaceBlit), new D3DSwToSurfaceScale(SurfaceType.IntArgb, 0), new D3DSwToSurfaceScale(SurfaceType.IntArgbPre, 1), new D3DSwToSurfaceScale(SurfaceType.IntRgb, 3), new D3DSwToSurfaceScale(SurfaceType.IntBgr, 4), new D3DSwToSurfaceScale(SurfaceType.ThreeByteBgr, 9), new D3DSwToSurfaceScale(SurfaceType.Ushort565Rgb, 5), new D3DSwToSurfaceScale(SurfaceType.Ushort555Rgb, 6), new D3DSwToSurfaceScale(SurfaceType.ByteIndexed, 7), new D3DSwToSurfaceTransform(SurfaceType.IntArgb, 0), new D3DSwToSurfaceTransform(SurfaceType.IntRgb, 3), new D3DSwToSurfaceTransform(SurfaceType.IntBgr, 4), new D3DSwToSurfaceTransform(SurfaceType.ThreeByteBgr, 9), new D3DSwToSurfaceTransform(SurfaceType.Ushort565Rgb, 5), new D3DSwToSurfaceTransform(SurfaceType.Ushort555Rgb, 6), new D3DSwToSurfaceTransform(SurfaceType.ByteIndexed, 7), d3DSwToSurfaceTransform, new D3DGeneralTransformedBlit(d3DSwToSurfaceTransform), new D3DTextureToSurfaceBlit(), new D3DTextureToSurfaceScale(), new D3DTextureToSurfaceTransform(), d3DSwToTextureBlit, new D3DSwToTextureBlit(SurfaceType.IntRgb, 3), new D3DSwToTextureBlit(SurfaceType.IntArgb, 0), new D3DSwToTextureBlit(SurfaceType.IntBgr, 4), new D3DSwToTextureBlit(SurfaceType.ThreeByteBgr, 9), new D3DSwToTextureBlit(SurfaceType.Ushort565Rgb, 5), new D3DSwToTextureBlit(SurfaceType.Ushort555Rgb, 6), new D3DSwToTextureBlit(SurfaceType.ByteIndexed, 7), new D3DGeneralBlit(D3DSurfaceData.D3DTexture, CompositeType.SrcNoEa, d3DSwToTextureBlit) });
    }
    
    private static int createPackedParams(boolean b, boolean b2, boolean b3, boolean b4, int n, int n2) {
        return n2 << 16 | n << 8 | (b2 ? 1 : 0) << 3 | (b3 ? 1 : 0) << 2 | (b4 ? 1 : 0) << 1 | (b ? 1 : 0) << 0;
    }
    
    private static void enqueueBlit(RenderQueue renderQueue, SurfaceData surfaceData, SurfaceData surfaceData2, int x, int x2, int x3, int x4, int x5, double x6, double x7, double x8, double x9) {
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacityAndAlignment(72, 24);
        buffer.putInt(31);
        buffer.putInt(x);
        buffer.putInt(x2).putInt(x3);
        buffer.putInt(x4).putInt(x5);
        buffer.putDouble(x6).putDouble(x7);
        buffer.putDouble(x8).putDouble(x9);
        buffer.putLong(surfaceData.getNativeOps());
        buffer.putLong(surfaceData2.getNativeOps());
    }
    
    static void Blit(SurfaceData ref, SurfaceData surfaceData, Composite comp, Region clip, AffineTransform xform, int n, int n2, int n3, int n4, int n5, double n6, double n7, double n8, double n9, int n10, boolean b) {
        int flags = 0;
        if (ref.getTransparency() == 1) {
            flags |= 0x1;
        }
        D3DSurfaceData d3DSurfaceData = (D3DSurfaceData)surfaceData;
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            instance.addReference(ref);
            if (b) {
                D3DContext.setScratchSurface(d3DSurfaceData.getContext());
            }
            else {
                BufferedContext.validateContext(d3DSurfaceData, d3DSurfaceData, clip, comp, xform, null, null, flags);
            }
            enqueueBlit(instance, ref, surfaceData, createPackedParams(false, b, false, xform != null, n, n10), n2, n3, n4, n5, n6, n7, n8, n9);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
        if (d3DSurfaceData.getType() == 1) {
            ((D3DScreenUpdateManager)ScreenUpdateManager.getInstance()).runUpdateNow();
        }
    }
    
    static void IsoBlit(SurfaceData surfaceData, SurfaceData surfaceData2, BufferedImage srcImg, BufferedImageOp bufferedImageOp, Composite comp, Region clip, AffineTransform xform, int n, int n2, int n3, int n4, int n5, double n6, double n7, double n8, double n9, boolean b) {
        int flags = 0;
        if (surfaceData.getTransparency() == 1) {
            flags |= 0x1;
        }
        D3DSurfaceData dstData = (D3DSurfaceData)surfaceData2;
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        boolean n10 = false;
        instance.lock();
        try {
            D3DSurfaceData d3DSurfaceData = (D3DSurfaceData)surfaceData;
            int type = d3DSurfaceData.getType();
            n10 = (type != 3);
            BufferedContext.validateContext(d3DSurfaceData, dstData, clip, comp, xform, null, null, flags);
            if (bufferedImageOp != null) {
                BufferedBufImgOps.enableBufImgOp(instance, d3DSurfaceData, srcImg, bufferedImageOp);
            }
            enqueueBlit(instance, surfaceData, surfaceData2, createPackedParams(true, b, (boolean)n10, xform != null, n, 0), n2, n3, n4, n5, n6, n7, n8, n9);
            if (bufferedImageOp != null) {
                BufferedBufImgOps.disableBufImgOp(instance, bufferedImageOp);
            }
        }
        finally {
            instance.unlock();
        }
        if (n10 && dstData.getType() == 1) {
            ((D3DScreenUpdateManager)ScreenUpdateManager.getInstance()).runUpdateNow();
        }
    }
}
